##################### generated by xml-casa (v2) from imagepol.xml ##################
##################### 3b7437629b45930681a8977ca52048f2 ##############################
from __future__ import absolute_import 
from .__casac__.imagepol import imagepol as _imagepol

from .errors import create_error_string
from .typecheck import CasaValidator as _validator
_pc = _validator( )
from .coercetype import coerce as _coerce
from .image import image as _wrap_image

class imagepol:
    _info_group_ = """images"""
    _info_desc_ = """Polarimetric analysis of images"""
    ### self
    def __init__(self, *args, **kwargs):
        """The constructor takes an image as its input.  This can be specified as
        either the name of a disk file, or you can give a pre-existing Image
        tool.   If you give a disk file name, the disk image file may
        be in casa, fits or Miriad format.
        
        The input image must have a Stokes axis.  The exact collection of Stokes
        that the image has, determines what the Imagepol tool can compute.
        Stokes I, Q, U, and V refer to total intensity, two components of linear
        polarization, and circular polatization, respectively.  Therefore, if
        you ask for linear polarization and the image only has Stokes I and V,
        you will get an error.
        
        The input image may contain data at many frequencies.  For example, the
        image may be a 4D image with axes RA, DEC, Stokes and Frequency (order
        not important) where the Frequency axis is regularly sampled.  However,
        the image may also contain many frequencies at irregular intervals.
        Such an image may be created with the Image tool constructor called
        imageconcat.  It enables you to concatenate
        images along an axis, and it allows irregular coordinate values along that
        axis.
        """
        self._swigobj = kwargs.get('swig_object',None)
        if self._swigobj is None:
            self._swigobj = _imagepol()

    def imagepoltestimage(self, outfile='imagepol.iquv', rm=[ float(0.0) ], pa0=float(0.0), sigma=float(0.01), nx=int(32), ny=int(32), nf=int(32), f0=float(1.4e9), bw=float(128.0e6)):
        """This function can be used to generate a test image and then
        attach the Imagepol tool to it.
        
        The test image is 4-dimensional (RA, DEC, Stokes and Frequency).  The
        Stokes axis holds I,Q,U and V.  The source is just a constant I (if you
        don't add noise all spatial pixels will be identical) and V.  Q and U
        vary with frequency according to the specified Rotation Measure
        components (no attempt to handle bandwidth smearing within channels is
        made).  The actual values of I,Q,U, and V are chosen arbitrarily
        otherwise (could be added as arguments if desired).
        
        You can use this image, in particular, to explore the Rotation Measure
        algorithms in functions rotationmeasure and
        fourierrotationmeasure.
        
        If you don't specify the Rotation Measure, then it is chosen for you so
        that there is no position angle ambiguity between adjacent channels
        (the value will be sent to the Logger).
        
        The noise added to the image is specified as a fraction of the total
        intensity (constant).  Gaussian noise with a standard deviation of
        {stfaf sigma * $I_{max}$} is then added to the image.
        """
        return self._swigobj.imagepoltestimage(outfile, rm, pa0, sigma, nx, ny, nf, f0, bw)

    def complexlinpol(self, outfile):
        """This function  produces
        the complex linear polarization; $Q+iU$ and writes
        it to a disk image file.
        
        The Image tool cannot yet handle Complex
        images.  You must therefore write the Complex image to disk.  The
        Viewer can display Complex images. Also the
        Table tool can access Complex images.
        """
        return self._swigobj.complexlinpol(outfile)

    def complexfraclinpol(self, outfile):
        """This function produces
        the complex fractional linear polarization; $(Q+iU)/I$ and writes
        it to a disk image file.
        
        The Image tool cannot yet handle Complex
        images.  You must therefore write the Complex image to disk.  The
        Viewer can display Complex images. Also the
        Table tool can access Complex images.
        """
        return self._swigobj.complexfraclinpol(outfile)

    def depolratio(self, infile, debias=False, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the linear
        depolarization ratio computed from two frequencies; this is the ratio of
        the fractional linear polarization at the two frequencies.  Generally
        this is done when you have generated two images, each at a different frequency
        (continuum work). Thus if the fractional linear polarization images are
        $m_{nu 1}$ and $m_{nu 2}$ then the depolarization ratio is
        $m_{nu 1}/ m_{nu 2}$.
        
        This function operates with two images; the first (at frequency $nu 1$)
        is the one attached to your Imagepol tool.  The second (at
        frequency $nu 2$) is supplied via the argument {stfaf infile}, which
        is  a String holding the name of the
        imagefile.
        
        In generating the depolarization ratio, you may optionally debias the
        linearly polarized intensity.  This requires the standard deviation of
        the thermal noise.  You can either supply it if you know it, or it will
        be worked out for you with outliers from the mean clipped at the
        specified level.
        
        You can get the depolarization ratio error image with function
        sigmadepolratio.
        """
        return _wrap_image(swig_object=self._swigobj.depolratio(infile, debias, clip, sigma, outfile))

    def close(self):
        """This function closes the imagepol tool.  This means that it detaches
        the tool from its imagefile (flushing all the changes first).  The
        imagepol tool is ``null'' after this change (it is not destroyed) and
        calling any toolfunction other than open will result in an
        error.
        """
        return self._swigobj.close()

    def done(self):
        """This function is the same as close().
        """
        return self._swigobj.done()

    def fourierrotationmeasure(self, complex='', amp='', pa='', real='', imag='', zerolag0=False):
        """This function  will only work if
        you attach the Imagepol tool (using open) to an image containing
        Stokes Q and U, and a regular frequency axis.  It Fourier transforms
        the complex linear polarization (Q+iU) over the spectral axis to the
        rotation measure axis.  Thus, if your input image contained
        RA/DEC/Stokes/Frequency, the output image would be
        RA/DEC/RotationMeasure.  The Rotation Measure axis has as many pixels
        as the spectral axis.
        
        This method enables you to see the polarization as a function of
        Rotation Meausure.  Its main use is when searching for large RMs.  See
        Killeen, Fluke, Zhao and Ekers (1999, preprint) for a description of
        this method (or http://www.atnf.csiro.au/verb+~+nkilleen/rm.ps) and its
        advantages over the traditional method
        (rotationmeasure) of
        extracting the Rotation Measure.
        
        Although you can write out the complex polarization image with the
        argument {stfaf complex}, you can't do much with it because Image
        tools cannot handle complex images.  Hence you can
        also write out the complex linear polarization image in any or all of
        the other forms.
        
        The argument {stfaf zerolag0} allows you to force the zero lag (or
        central bin) of the Rotation Measure spectrum to zero (effectively by
        subtracting the mean of Q and U from the Q and U images).  This may
        avoid Gibbs phenomena from any strong low Rotation Measure signal which
        would normally fall into the central bin.
        """
        return self._swigobj.fourierrotationmeasure(complex, amp, pa, real, imag, zerolag0)

    def fraclinpol(self, debias=False, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function
        returns the fractional linear polarization; $sqrt{(Q^2+U^2)}/I$.
        
        You may optionally debias the polarized intensity.  This requires the
        standard deviation of the thermal noise.  You can either supply it if
        you know it, or it will be worked out for you with outliers from the
        mean clipped at the specified level.
        """
        return _wrap_image(swig_object=self._swigobj.fraclinpol(debias, clip, sigma, outfile))

    def fractotpol(self, debias=False, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function
        returns the fractional linear polarization; $sqrt{(Q^2+U^2+V^2)}/I$.
        
        You may optionally debias the polarized intensity.  This requires the
        standard deviation of the thermal noise.  You can either supply it if
        you know it, or it will be worked out for you with outliers from the
        mean clipped at the specified level.
        
        If your image contains only Q and U, or only V, then just
        those components contribute to the total polarized intensity.
        """
        return _wrap_image(swig_object=self._swigobj.fractotpol(debias, clip, sigma, outfile))

    def linpolint(self, debias=False, clip=float(10.0), sigma=float(-1), outfile='', region='', mask='', stretch=False):
        """This application returns the linearly polarized intensity; $sqrt{(Q^2+U^2)}$.
        
        The linearly polarized intensity may optionally be debiased (if debias=True).
        This requires an estimate of the thermal noise level (sigma). The resulting
        image will be computed using
        
        $sqrt{(Q^2 + U^2 - sigma^2)}$
        
        If the specified value of sigma is positive, that is the value that will
        be used for debiasing. If it is not, the value used for sigma is computed
        using the following algorithm:
        
        1. First, a stokes plane(s) is chosen on which the sigma computation is performed.
        If the V plane exists, that is used to compute sigma. If not, then the value of
        sigma is computed to be the average of the sigma values of the Q and U planes.
        
        2. For the relevant plane(s), the sigma value is computed for unmasked pixel values
        which lie within +/- (clip * stddev) values of the mean for that plane, where the mean
        and stddev (the standard deviation) are computed by using all unmasked pixel values
        in the relevant plane. In this way, outliers (eg pixels due to possible
        astronomical signals) are excluded in the computation of sigma.
        
        In the output image, pixels for which the expression inside the square root is
        negative are masked, and their values are set to zero.
        
        If a region and/or mask is specified, debias=True and sigma is not positive so
        that the value of sigma is determined using the algorithm above, the region and mask
        are applied first to the plane(s) from which the value of sigma is determined and
        then sigma is computed.
        """
        return _wrap_image(swig_object=self._swigobj.linpolint(debias, clip, sigma, outfile, region, mask, stretch))

    def linpolposang(self, outfile='', region='', mask='', stretch=False):
        """This function returns the linearly polarized position angle image 0.5*arctan(U/Q) in degrees.
        The output image will not have a restoring beam, even if the input image does.
        """
        return _wrap_image(swig_object=self._swigobj.linpolposang(outfile, region, mask, stretch))

    def makecomplex(self, complex, real='', imag='', amp='', phase=''):
        """This function generates a Complex imagefile from either
        a real and imaginary, or an amplitude and phase pair of images.
        If you give a linear position angle image for the phase,
        it will be multipled by two before the real and imaginary
        parts are formed.
        """
        return self._swigobj.makecomplex(complex, real, imag, amp, phase)

    def open(self, image={ }):
        """Before polarimetric analysis can commence, an imagefile must be
        attached to the imagepol tool using the open function.  Also, use this
        function when you are finished analyzing the current imagefile and
        want to attach to another one.  This function detaches the imagetool
        from the current imagefile, if one exists, and reattaches it (opens)
        to the new imagefile.
        
        The input image file may be in native casa, fits, or Miriad
        format.  Look htmlref{here}{IMAGES:FOREIGNIMAGES}  for more
        information on foreign images.
        
        The input image must have a Stokes axis. The exact collection of
        Stokes that the image has, determines what the Imagepol tool can
        compute. Stokes I, Q, U, and V refer to total intensity, two
        components of linear polarization, and circular polatization,
        respectively. Therefore, if you ask for linear polarization and the
        image only has Stokes I and V, you will get an error.
        
        The input image may contain data at many frequencies. For example, the
        image may be a 4D image with axes RA, DEC, Stokes and Frequency (order
        not important) where the Frequency axis is regularly sampled. However,
        the image may also contain many frequencies at irregular
        intervals. Such an image may be created with the Image tool function
        imageconcat. It enables you to concatenate images along an axis, and
        it allows irregular coordinate values along that axis.
        """
        return self._swigobj.open(image)

    def pol(self, which, debias=False, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function just packages the other specific polarization
        functions into one where you specify an operation with the
        argument {stfaf which} (can be useful for scripts).
        This argument can take the values:
        
        begin{itemize}
        item 'lpi' - linearly polarized intensity (function
        linpolint)
        
        item 'tpi' - total polarized intensity (function
        totpolint)
        
        item 'lppa' linearly polarized position angle (function
        linpolposang)
        
        item 'flp' - fractional linear polarization (function
        fraclinpol)
        
        item 'ftp' - fractional total polarized intensity (function
        fractotpol)
        
        end{itemize}
        """
        return _wrap_image(swig_object=self._swigobj.pol(which, debias, clip, sigma, outfile))

    def rotationmeasure(self, rm='', rmerr='', pa0='', pa0err='', nturns='', chisq='', sigma=float(-1), rmfg=float(0.0), rmmax=float(0.0), maxpaerr=float(1e30), plotter='', nx=int(5), ny=int(5)):
        """This function generates the rotation
        measure image from a collection of different frequencies.  It will only
        work if the Imagepol tool is attached to an image containing
        Stokes $Q$ and $U$, and a frequency axis (regular or irregular) with at
        least 2 pixels.  It will work out the position angle images for you.
        
        See also the fourierrotationmeasure
        function for a new Fourier-based approach.
        
        Rotation Measure algorithms that work robustly are not common.  The main
        problem is in trying to account for the $n- pi$ ambiguity (see Leahy et
        al, Astronomy & Astrophysics, 156, 234 or Killeen et al;
        http://www.atnf.csiro.au/verb+~+nkilleen/rm.ps).
        
        The algorithm that this function uses is that of Leahy et al. (see Appendix A.1).  But as in all
        these algorithms, the basic process is that for each spatial pixel, a
        vector of position angles (i.e.  at the different frequencies) is fit to
        determine the rotation measure and the position angle at zero wavelength
        (and their errors).   An image containing
        the number of $n- pi$ turns that were added to the data
        at each spatial pixel and for which the best fit was found can be written.
        The reduced chi-squared image for the fits can also be written.
        
        Note that no assessment of curvature (i.e. deviation
        from the simple linear position angle - $lambda^2$ functional form)
        is made.
        
        Any combination of output images can be written.
        
        The argument {stfaf sigma} gives the thermal noise in Stokes Q and U.
        By default it is determined automatically using the image data.  But if it proves
        to be inaccurate (maybe not many signal-free pixels), it may be specified.
        This is used for calculating the error in the
        position angles (propagation of Gaussian errors).
        
        The argument {stfaf maxpaerr} specifies the maximum allowable error in
        the position angle that is acceptable.  The default is an infinite
        value.  From the standard propagation of errors, the error in the
        linearly polarized position angle is determined from the Stokes $Q$ and
        $U$ images (at each spatial pixel for each frequency).  At each spatial
        pixel we do a fit to the position angle vector (i.e.  at the different
        frequencies) to determine the rotation measure.  If the position angle
        error for any pixel in the vector exceeds the specified value, it is
        dropped from the fit.     The process generates an error for the
        fit and this is used to compute the errors in the output
        images.
        
        Note that {stfaf maxpaerr} is {it not} used to specify that any pixel
        for which the output position angle error exceeds this value
        should be masked out.
        
        The argument {stfaf rmfg} is used to specify a foreground RM value.  For
        example, you may know the mean RM in some direction out of the Galaxy,
        then including this can aid the algorithm by reducing ambiguity.
        
        The argument {stfaf rmmax} specifies the maximum absolute RM that
        should be solved for.  This quite an important parameter.  If you leave
        it at the default, zero, no ambiguity handling will be
        used.  So some apriori information should be supplied; this
        is the basic problem with rotation measure algorithms.
        """
        return self._swigobj.rotationmeasure(rm, rmerr, pa0, pa0err, nturns, chisq, sigma, rmfg, rmmax, maxpaerr, plotter, nx, ny)

    def sigma(self, clip=float(10.0)):
        """This function returns the standard deviation from V, Q&U or I in that
        order of precedence.  It is attempting to give you the best estimate of
        the thermal noise it can from the data.  Outliers from the mean are
        clipped at the specified level.
        """
        return self._swigobj.sigma(clip)

    def sigmadepolratio(self, infile, debias=False, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the error
        in the linear depolarization ratio computed from two frequencies; this
        is the ratio of the fractional linear polarization at the two
        frequencies.  Generally this is done when you have generated two
        images, each at a different frequency (continuum work). Thus if the
        fractional linear polarzation images are $m1$ and $m2$ then the
        depolarization ratio is $m1/m2$.
        
        This function operates with two images; the first is attached
        to the Imagepol tool.  The second is supplied via the
        argument {stfaf infile}, which is  a String
        holding the name of the imagefile.
        
        In generating the depolarization ratio, and hence its error, you may
        optionally debias the  linearly polarized intensity.  This requires the
        standard deviation of the thermal noise.  You can either supply it if
        you know it, or it will be worked out for you with outliers from the
        mean clipped at the specified level.
        
        You can get the depolarization ratio image with function
        depolratio.
        """
        return _wrap_image(swig_object=self._swigobj.sigmadepolratio(infile, debias, clip, sigma, outfile))

    def sigmafraclinpol(self, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the
        error (standard deviation) of the fractional linear polarization.
        This result comes from standard propagation of errors.  The result is
        an on-the-fly Image tool as the error is signal-to-noise ratio
        dependent.
        
        This function requires the standard deviation of the thermal noise.  You
        can either supply it if you know it, or it will be worked out for you
        with outliers from the mean clipped at the specified level.
        """
        return _wrap_image(swig_object=self._swigobj.sigmafraclinpol(clip, sigma, outfile))

    def sigmafractotpol(self, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the
        error (standard deviation) of the fractional total polarization.  This
        result comes from standard propagation of errors.  The result is an
        on-the-fly Image tool as the error is signal-to-noise ratio dependent.
        
        This function requires the standard deviation of the thermal noise.  You
        can either supply it if you know it, or it will be worked out for you
        with outliers from the mean clipped at the specified level.
        """
        return _wrap_image(swig_object=self._swigobj.sigmafractotpol(clip, sigma, outfile))

    def sigmalinpolint(self, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the error (standard
        deviation) of the linearly polarized intensity; $sqrt{(Q^2+U^2)}$.
        This result comes from standard propagation of statistical errors.
        The result is a float as the error is not signal-to-noise
        ratio dependent
        
        This function requires the standard deviation of the thermal noise.  You
        can either supply it if you know it, or it will be worked out for you
        with outliers from the mean clipped at the specified level.
        """
        return self._swigobj.sigmalinpolint(clip, sigma, outfile)

    def sigmalinpolposang(self, clip=float(10.0), sigma=float(-1), outfile=''):
        """This function  returns the
        error (standard deviation) of the linearly polarized position angle
        ($0.5 tan^{-1}(U/Q)$$sqrt{(Q^2+U^2)}$) in degrees.  This result
        comes from standard propagation of errors.  The result is an
        on-the-fly Image tool as the error is signal-to-noise ratio dependent.
        
        This function requires the standard deviation of the thermal noise.  You
        can either supply it if you know it, or it will be worked out for you
        with outliers from the mean clipped at the specified level.
        """
        return _wrap_image(swig_object=self._swigobj.sigmalinpolposang(clip, sigma, outfile))

    def sigmastokes(self, which, clip=float(10.0)):
        """This function  returns the standard
        deviation of the noise for the specified Stokes.  Outliers from the mean
        are clipped at the specified level.
        """
        return self._swigobj.sigmastokes(which, clip)

    def sigmastokesi(self, clip=float(10.0)):
        """This function  returns the standard deviation of the noise for the
        Stokes I data.  Outliers from the mean are clipped at the specified
        level.
        """
        return self._swigobj.sigmastokesi(clip)

    def sigmastokesq(self, clip=float(10.0)):
        """This function  returns the standard deviation of the noise for the
        Stokes Q data.  Outliers from the mean are clipped at the specified
        level.
        """
        return self._swigobj.sigmastokesq(clip)

    def sigmastokesu(self, clip=float(10.0)):
        """This function  returns the standard
        deviation of the noise for the Stokes U data.  Outliers from the mean
        are clipped at the specified level.
        """
        return self._swigobj.sigmastokesu(clip)

    def sigmastokesv(self, clip=float(10.0)):
        """This function  returns the standard
        deviation of the noise for the Stokes V data.  Outliers from the mean
        are clipped at the specified level.
        """
        return self._swigobj.sigmastokesv(clip)

    def sigmatotpolint(self, clip=float(10.0), sigma=float(-1)):
        """This function  returns the error (standard
        deviation) of the total polarized intensity; $sqrt{(Q^2+U^2+V^2)}$.
        This result comes from standard propagation of statistical errors.
        The result is a float as the error is not signal-to-noise
        ratio dependent
        
        This function requires the standard deviation of the thermal noise.  You
        can either supply it if you know it, or it will be worked out for you
        with outliers from the mean clipped at the specified level.
        """
        return self._swigobj.sigmatotpolint(clip, sigma)

    def stokes(self, which, outfile=''):
        """This function returns an on-the-fly image tool containing the
        specified Stokes only.  This interface can be useful for scripts.
        """
        return _wrap_image(swig_object=self._swigobj.stokes(which, outfile))

    def stokesi(self, outfile=''):
        """This function returns an on-the-fly image tool containing Stokes I only.
        """
        return _wrap_image(swig_object=self._swigobj.stokesi(outfile))

    def stokesq(self, outfile=''):
        """This function returns an on-the-fly image tool containing Stokes Q only.
        """
        return _wrap_image(swig_object=self._swigobj.stokesq(outfile))

    def stokesu(self, outfile=''):
        """This function returns an on-the-fly image tool containing Stokes U only.
        """
        return _wrap_image(swig_object=self._swigobj.stokesu(outfile))

    def stokesv(self, outfile=''):
        """This function returns an on-the-fly image tool containing Stokes V only.
        """
        return _wrap_image(swig_object=self._swigobj.stokesv(outfile))

    def summary(self):
        """This function just lists a summary of the Imagepol tool to the logger.
        Currently it just summarizes the image to which the tool is attached.
        """
        return self._swigobj.summary()

    def totpolint(self, debias=False, clip=float(10.0), sigma=float(-1), outfile='', region='', mask='', stretch=False):
        """This application returns the total polarized intensity;
        
        $sqrt{( Q^2+ U^2+ V^2)}$.
        
        If the image contains only Q and U, or only V, then just
        those components contribute to the total polarized intensity.
        
        The polarized intensity may optionally be debiased (if debias=True).
        This requires an estimate of the thermal noise level (sigma).
        The resulting image will be computed using
        
        $sqrt{(Q^2 + U^2 + V^2 - sigma^2)}$
        
        If the specified value of sigma is positive, that is the value that will
        be used for debiasing. If it is not, the value used for sigma is computed
        using the following algorithm:
        
        1. First, a stokes plane(s) is chosen on which the sigma computation is performed.
        If the V plane exists, that is used to compute sigma. If not, then the value of
        sigma is computed to be the average of the sigma values of the Q and U planes.
        
        2. For the relevant plane(s), the sigma value is computed for unmasked pixel values
        which lie within +/- (clip * stddev) values of the mean for that plane, where the mean
        and stddev (the standard deviation) are computed by using all unmasked pixel values
        in the relevant plane. In this way, outliers (eg pixels due to possible
        astronomical signals) are excluded in the computation of sigma.
        
        In the output image, pixels for which the expression inside the square root is
        negative are masked, and their values are set to zero.
        
        If a region and/or mask is specified, debias=True and sigma is not positive so
        that the value of sigma is determined using the algorithm above, the region and mask
        are applied first to the plane(s) from which the value of sigma is determined and
        then sigma is computed.
        """
        return _wrap_image(swig_object=self._swigobj.totpolint(debias, clip, sigma, outfile, region, mask, stretch))

